# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContentType',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('contenttypes.contenttype',),
        ),
    ]



from rest_framework.generics import ListAPIView
from wq.db.patterns.identify.filters import IdentifierFilterBackend
from .models import FilterableModel
from wq.db.rest.serializers import ModelSerializer


class FilterableView(ListAPIView):
    serializer_class = ModelSerializer.for_model(
        FilterableModel,
        include_fields='__all__',
    )
    filter_backends = [IdentifierFilterBackend]
    queryset = FilterableModel.objects.all()

    router = None
    action = None

    def filter_by_identifiedmodel(self, queryset, ids):
        return queryset.filter(
            parent_id__in=ids,
        )



from rest_framework.filters import BaseFilterBackend
from django.core.exceptions import FieldDoesNotExist


class FilterBackend(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        kwargs = {}
        for key, val in list(view.kwargs.items()) + list(request.GET.items()):
            if key in getattr(view, 'ignore_kwargs', []):
                continue
            if isinstance(val, list):
                kwargs[key] = val[0]
            else:
                kwargs[key] = val
        model = getattr(view, 'model', None) or queryset.model

        for key, val in list(kwargs.items()):
            field_name = key.split('__')[0]
            try:
                field = model._meta.get_field(field_name)
            except FieldDoesNotExist:
                del kwargs[key]
                continue

            if field_name != key:
                continue

            rel = field.remote_field
            if not rel:
                continue

            pcls = rel.model
            router = getattr(view, 'router', None)
            if router:
                slug = router.get_lookup_for_model(pcls)
                kwargs[key] = pcls.objects.get(**{slug: val})
            else:
                kwargs[key] = pcls.objects.get(pk=val)

        return queryset.filter(**kwargs)



from rest_framework.permissions import BasePermission
from .model_tools import get_ct


class ModelPermissions(BasePermission):
    METHOD_PERM = {
        'GET': 'view',
        'HEAD': 'view',
        'OPTIONS': 'view',
        'POST': 'add',
        'PUT': 'change',
        'PATCH': 'change',
        'DELETE': 'delete',
    }

    def has_permission(self, request, view):
        if getattr(view, 'model', None) is None:
            return True
        user = request.user
        ct = get_ct(view.model)
        result = has_perm(user, ct, self.METHOD_PERM[request.method])
        return result


def has_perm(user, ct, perm):
    if perm == 'view':
        return True
    if isinstance(ct, str):
        perm = '%s_%s' % (ct, perm)
    else:
        perm = '%s.%s_%s' % (ct.app_label, perm, ct.model)

    if user.is_authenticated:
        return user.has_perm(perm)
    else:
        from django.conf import settings
        return perm in getattr(settings, 'ANONYMOUS_PERMISSIONS', {})



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-27 17:35
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ModelWithNaturalKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('note', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='NaturalKeyChild',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField()),
            ],
        ),
        migrations.CreateModel(
            name='NaturalKeyParent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField()),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='naturalkeyparent',
            unique_together=set([('slug',)]),
        ),
        migrations.AddField(
            model_name='naturalkeychild',
            name='parent',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='naturalkey_app.NaturalKeyParent'),
        ),
        migrations.AddField(
            model_name='modelwithnaturalkey',
            name='key',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='naturalkey_app.NaturalKeyChild'),
        ),
        migrations.AlterUniqueTogether(
            name='naturalkeychild',
            unique_together=set([('parent', 'date')]),
        ),
    ]



from rest_framework.generics import GenericAPIView as RestGenericAPIView
from rest_framework.response import Response
from rest_framework.decorators import action
from rest_framework import status, viewsets
from .model_tools import get_ct, get_object_id, get_by_identifier
from django.core.exceptions import FieldDoesNotExist
from django.db.models import ProtectedError


class GenericAPIView(RestGenericAPIView):
    router = None
    ignore_kwargs = []

    @property
    def template_name(self):
        """
        Infer template name from view/viewset name
        """
        name = type(self).__name__
        name = name.replace('ViewSet', '')
        name = name.replace('View', '')
        return name.lower() + '.html'

    @property
    def depth(self):
        return 0

    def get_template_names(self):
        return [self.template_name]

    def get_queryset(self):
        if self.router is not None and self.model is not None:
            return self.router.get_queryset_for_model(self.model, self.request)
        return super(GenericAPIView, self).get_queryset()

    def get_serializer_class(self):
        if self.router is not None and self.model is not None:
            return self.router.get_serializer_for_model(self.model, self.depth)
        return super(GenericAPIView, self).get_serializer_class()

    def head(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)


class SimpleView(GenericAPIView):
    def get(self, request, *args, **kwargs):
        return Response({})


class SimpleViewSet(viewsets.ViewSet, GenericAPIView):
    def list(self, request, *args, **kwargs):
        return Response({})


class ModelViewSet(viewsets.ModelViewSet, GenericAPIView):
    target = None

    @property
    def template_name(self):
        basename = get_ct(self.model).identifier
        if self.action in ('retrieve', 'create', 'update', 'delete'):
            suffix = 'detail'
        elif self.action and self.action != 'metadata':
            suffix = self.action
        else:
            suffix = 'list'
        return "%s_%s.html" % (basename, suffix)

    @property
    def depth(self):
        if self.action in ('retrieve', 'edit'):
            return 1
        else:
            return 0

    @action(detail=True)
    def edit(self, request, *args, **kwargs):
        """
        Generates a context appropriate for editing a form
        """
        response = self.retrieve(request, *args, **kwargs)
        obj = self.get_object()
        serializer = self.get_serializer(obj)
        serializer.add_lookups(response.data)
        return response

    def new(self, request):
        """
        new is a variant of the "edit" action, but with no existing model
        to lookup.
        """
        self.action = 'edit'
        init = request.GET.dict()
        for arg in self.ignore_kwargs:
            init.pop(arg, None)
        for key in list(init.keys()):
            try:
                field = self.model._meta.get_field(key)
            except FieldDoesNotExist:
                del init[key]
            else:
                if field.remote_field:
                    fk_model = field.remote_field.model
                    try:
                        obj = get_by_identifier(fk_model.objects, init[key])
                    except fk_model.DoesNotExist:
                        del init[key]
                    else:
                        init[key] = obj.pk

        obj = self.model(**init)
        serializer = self.get_serializer(obj)
        data = serializer.data
        serializer.add_lookups(data)
        return Response(data)

    def retrieve(self, request, *args, **kwargs):
        """
        Custom retrieve watches for "new" lookup value and switches modes
        accordingly
        """
        lookup = self.lookup_url_kwarg or self.lookup_field

        if self.kwargs.get(lookup, "") == "new":
            # new/edit mode
            return self.new(request)
        else:
            # Normal detail view
            return super(ModelViewSet, self).retrieve(request, *args, **kwargs)

    def list(self, request, *args, **kwargs):
        response = super(ModelViewSet, self).list(
            request, *args, **kwargs
        )
        if not isinstance(response.data, dict):
            return response

        if self.target:
            response.data['target'] = self.target
        ct = get_ct(self.model)
        for pct, fields in ct.get_foreign_keys().items():
            if len(fields) == 1:
                self.get_parent(pct, fields[0], response)
        return response

    def create(self, request, *args, **kwargs):
        response = super(ModelViewSet, self).create(
            request, *args, **kwargs
        )
        if not request.accepted_media_type.startswith('text/html'):
            # JSON request, assume client will handle redirect
            return response

        # HTML request, probably a form post from an older browser
        if response.status_code == status.HTTP_201_CREATED:
            return self.postsave(request, response)
        else:
            return self.saveerror(request, response)

    def update(self, request, *args, **kwargs):
        response = super(ModelViewSet, self).update(
            request, *args, **kwargs
        )
        if not request.accepted_media_type.startswith('text/html'):
            # JSON request, assume client will handle redirect
            return response

        # HTML request, probably a form post from an older browser
        if response.status_code == status.HTTP_200_OK:
            return self.postsave(request, response)
        else:
            return self.saveerror(request, response)

    def postsave(self, request, response):
        ct = get_ct(self.model)
        conf = ct.get_config(request.user)

        # Redirect to new page
        postsave = conf.get('postsave', ct.identifier + '_detail')
        if '_' in postsave:
            page, mode = postsave.split('_')
        else:
            page = postsave
            mode = 'detail'

        oid = ""
        if page != ct.identifier and self.router:
            # Optional: return to detail view of a parent model
            pconf = self.router.get_page_config(page)
            if pconf.get('list', None) and mode != "list":
                oid = response.data.get(page + '_id', None)
        else:
            # Default: return to detail view of the saved model
            pconf = conf
            if mode != "list":
                oid = response.data['id']

        url = "/" + pconf['url']
        if pconf['url'] and pconf.get('list', None):
            url += "/"
        if oid:
            url += str(oid)
            if mode == "edit":
                url += "/edit"

        return Response(
            {'detail': 'Created'},
            status=status.HTTP_302_FOUND,
            headers={'Location': url}
        )

    def saveerror(self, request, response):
        errors = [{
            'field': key,
            'errors': val
        } for key, val in response.data.items()]
        template = get_ct(self.model).identifier + '_error.html'
        return Response(
            {
                'errors': errors,
                'post': request.DATA
            },
            status=response.status_code,
            template_name=template
        )

    def destroy(self, request, *args, **kwargs):
        try:
            response = super(ModelViewSet, self).destroy(
                request, *args, **kwargs
            )
        except ProtectedError as e:
            return Response({
                'non_field_errors': [
                    e.args[0]
                ]
            }, status=400)
        else:
            return response

    def get_parent(self, ct, kwarg_name, response):
        pid = self.kwargs.get(kwarg_name, None)
        if not pid:
            return

        pcls = ct.model_class()
        if self.router:
            slug = self.router.get_lookup_for_model(pcls)
            parent = pcls.objects.get(**{slug: pid})
        else:
            parent = get_by_identifier(pcls.objects, pid)
        if ct.urlbase == '':
            urlbase = ''
        else:
            urlbase = ct.urlbase + '/'
        objid = get_object_id(parent)
        response.data['parent_label'] = str(parent)
        response.data['parent_id'] = objid
        response.data['parent_url'] = '%s%s' % (urlbase, objid)
        response.data['parent_is_' + ct.identifier] = True
        response.data['parent_page'] = ct.identifier
        response.data['page_config'] = get_ct(self.model).get_config()
        if self.router:
            response.data['parent'] = self.router.serialize(
                parent,
                request=self.request,
            )
        return parent



from django.utils.module_loading import autodiscover_modules
from .routers import ModelRouter, router


__all__ = (
    "ModelRouter",
    "router",
    "autodiscover",
    "default_app_config",
)


def autodiscover():
    autodiscover_modules('rest', register_to=None)


default_app_config = 'wq.db.rest.apps.RestConfig'



from rest_framework.filters import BaseFilterBackend
from .models import Identifier


class IdentifierFilterBackend(BaseFilterBackend):
    ignore_extra = True
    exclude_apps = []
    view_kwarg = 'ids'

    @property
    def filter_options(self):
        if hasattr(self, '_filter_options'):
            return self._filter_options

        slugs = self.view.kwargs[self.view_kwarg].split('/')
        id_map, unresolved = Identifier.objects.resolve(
            slugs, exclude_apps=self.exclude_apps
        )
        options = {}
        if unresolved:
            options['extra'] = []
            for key, items in unresolved.items():
                items = [
                    item for item in items
                    if self.can_filter(item.content_type.model)
                ]
                if len(items) > 1:
                    raise Exception(
                        "Could not resolve %s to a single item!" % key
                    )
                elif len(items) == 1:
                    if id_map is None:
                        id_map = {}
                    id_map[key] = items[0]
                else:
                    options['extra'].append(key)

        if id_map:
            for slug, ident in id_map.items():
                ctype = ident.content_type.model
                if ctype not in options:
                    options[ctype] = []
                options[ctype].append(ident)

        self._filter_options = options
        return options

    def filter_queryset(self, request, queryset, view):
        self.request = request
        self.view = view

        # Filter by identifiers in url
        for name, idents in self.filter_options.items():
            if name == "extra":
                continue
            ids = [ident.object_id for ident in idents]
            fn = self.get_filter_by(name)
            if fn:
                queryset = fn(queryset, ids)
            else:
                raise Exception("Don't know how to filter by %s" % name)

        # Filter by any extra slugs in url
        if 'extra' in self.filter_options:
            fn = getattr(self.view, 'filter_by_extra', self.filter_by_extra)
            queryset = fn(queryset, self.filter_options['extra'])

        return queryset

    def get_filter_by(self, name):
        fn = getattr(self, 'filter_by_%s' % name, None)
        if not fn:
            fn = getattr(self.view, 'filter_by_%s' % name, None)
        return fn

    def can_filter(self, name):
        return self.get_filter_by(name) is not None

    def filter_by_extra(self, queryset, extra):
        if self.ignore_extra:
            return queryset
        else:
            raise Exception("Extra URL options found: %s" % extra)



import os

from wq.db.default_settings import (  # NOQA
    TEMPLATES,
    REST_FRAMEWORK,
    SRID,
)

SECRET_KEY = '1234'

MIDDLEWARE_CLASSES = (
    'django.middleware.csrf.CsrfViewMiddleware',
)

INSTALLED_APPS = (
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'wq.db.rest',
    'wq.db.rest.auth',
    'wq.db.patterns.identify',
    'tests.rest_app',
    'tests.conflict_app',
    'tests.patterns_app',
    'tests.naturalkey_app',
)

WITH_GIS = os.environ.get("WITH_GIS")

if WITH_GIS:
    INSTALLED_APPS += ('tests.gis_app',)


WITH_NONROOT = os.environ.get("WITH_NONROOT")


if os.environ.get('PSYCOPG2'):
    if WITH_GIS:
        engine = 'django.contrib.gis.db.backends.postgis'
    else:
        engine = 'django.db.backends.postgresql'
    DATABASES = {
        'default': {
            'ENGINE': engine,
            'NAME': 'wqdb_test',
            'USER': 'postgres',
        }
    }
else:
    if WITH_GIS:
        engine = 'django.contrib.gis.db.backends.spatialite'
        SPATIALITE_LIBRARY_PATH = 'mod_spatialite.so'
    else:
        engine = 'django.db.backends.sqlite3'
    DATABASES = {
        'default': {
             'ENGINE': engine,
             'NAME': 'wqdb_test.sqlite3',
        }
    }

USE_TZ = True
TIME_ZONE = "America/Chicago"

ROOT_URLCONF = "tests.urls"
BASE_DIR = os.path.dirname(__file__)
MEDIA_ROOT = os.path.join(BASE_DIR, "media")
VERSION_TXT = os.path.join(BASE_DIR, "version.txt")
WQ_SCRIPT_FILE = os.path.join(BASE_DIR, "index.html")

TEMPLATES[0]['DIRS'] += (os.path.join(BASE_DIR, "templates"),)

DEBUG = True



from rest_framework.test import APITestCase, APIClient
from django.conf import settings


class APIClient(APIClient):
    def generic(self, method, path, *args, **kwargs):
        if settings.WITH_NONROOT:
            path = '/wqsite' + path
        return super().generic(method, path, *args, **kwargs)


class APITestCase(APITestCase):
    client_class = APIClient



from .base import APITestCase
from rest_framework import status
from tests.rest_app.models import (
    RootModel, OneToOneModel, ForeignKeyModel, ExtraModel, UserManagedModel,
    Parent, Child, ItemType, Item, SlugModel, SlugRefParent, ChoiceModel,
)
from django.contrib.auth.models import User
from django.conf import settings


class TemplateTestCase(APITestCase):
    def setUp(self):
        instance = RootModel.objects.create(
            slug='instance',
            description="Test"
        )
        for cls in OneToOneModel, ForeignKeyModel, ExtraModel:
            cls.objects.create(
                root=instance,
            )
        user = User.objects.create(username="testuser", is_superuser=True)
        self.client.force_authenticate(user)
        UserManagedModel.objects.create(id=1, user=user)
        parent = Parent.objects.create(name="Test", pk=1)
        parent.children.create(name="Test 1")
        parent.children.create(name="Test 2")
        itype = ItemType.objects.create(name="Test", pk=1)
        itype.item_set.create(name="Test 1")
        itype.item_set.create(name="Test 2")
        slugref = SlugModel.objects.create(
            code="test",
            name="Test",
        )
        SlugRefParent.objects.create(
            ref=slugref,
            pk=1,
            name="Test Slug Ref"
        )
        SlugRefParent.objects.create(
            ref=SlugModel.objects.create(
                code="other",
                name="Other",
            ),
            pk=2,
            name="Test Another Ref",
        )
        ItemType.objects.create(
            name="Inactive",
            pk=2,
            active=False
        )
        ChoiceModel.objects.create(
            name="Test",
            pk=1,
            choice="two"
        )

    def assertHTMLEqual(self, expected_html, html, auto_replace=True):
        if settings.WITH_NONROOT and auto_replace:
            html = html.replace('/wqsite/', '/')
        super().assertHTMLEqual(expected_html, html)

    def check_html(self, url, expected_html):
        response = self.client.get(url)
        self.assertTrue(status.is_success(response.status_code), response.data)
        html = response.content.decode('utf-8')
        self.assertHTMLEqual(expected_html, html)

    # Test url="" use case
    def test_template_list_at_root(self):
        self.check_html("/", """
            <ul>
              <li><a href="/instance">instance</a></li>
            </ul>
        """)

    def test_template_detail_at_root(self):
        instance = RootModel.objects.get(slug='instance')
        self.check_html("/instance", """
            <h1>instance</h1>
            <p>Test</p>
            <h3>OneToOneModel</h3>
            <p>
              <a href="/onetoonemodels/{onetoone_pk}">
                onetoonemodel for instance
              </a>
            </p>
            <h3>ExtraModels</h3>
            <ul>
              <li>
                <a href="/extramodels/{extra_pk}">
                  extramodel for instance
                </a>
              </li>
            </ul>
            <p><a href="/instance/edit">Edit</a></p>
        """.format(
            onetoone_pk=instance.onetoonemodel.pk,
            extra_pk=instance.extramodels.all()[0].pk,
        ))

    def test_template_filter_by_parent(self):
        childs = Parent.objects.get(pk=1).children.order_by('pk')
        self.check_html('/parents/1/children', """
            <p>2 Records</p>
            <h3>Childs for <a href="/parents/1">Test</a></h3>
            <ul>
              <li><a href="/children/{c1_pk}">Test 1</a></li>
              <li><a href="/children/{c2_pk}">Test 2</a></li>
            </ul>
        """.format(
            c1_pk=childs[0].pk,
            c2_pk=childs[1].pk,
        ))

        items = ItemType.objects.get(pk=1).item_set.order_by('pk')
        self.check_html('/itemtypes/1/items', """
            <h3><a href="/itemtypes/1">Test</a> Items</h3>
            <ul>
              <li><a href="/items/{i1_pk}">Test 1</a></li>
              <li><a href="/items/{i2_pk}">Test 2</a></li>
            </ul>
        """.format(
            i1_pk=items[0].pk,
            i2_pk=items[1].pk,
        ))

    def test_template_detail_user_serializer(self):
        self.check_html('/usermanagedmodels/1', """
            <h1>Object #1</h1>
            <p>Created by testuser</p>
            <p></p>
        """)

    def test_template_custom_lookup(self):
        self.check_html('/slugmodels/test', "<h1>Test</h1>")

    def test_template_default_per_page(self):
        parent = Parent.objects.get(pk=1)
        parent.name = "Test 1"
        parent.save()
        for i in range(2, 101):
            Parent.objects.create(
                id=i,
                name="Test %s" % i,
            )

        html = """
            <p>100 Records</p>
            <div>
              <h3>Page 1 of 2</h3>
              <a href="http://testserver/parents/?page=2">Next 50</a>
            </div>
            <ul>
        """
        for i in range(1, 51):
            html += """
                <li><a href="/parents/{pk}">Test {pk}</a></li>
            """.format(pk=i)
        html += """
           </ul>
        """
        self.check_html("/parents/", html)

    def test_template_custom_per_page(self):
        for i in range(3, 102):
            child = Child.objects.create(
                name="Test %s" % i,
                parent_id=1,
            )

        self.check_html("/children/?page=2", """
           <p>101 Records</p>
           <div>
             <a href="http://testserver/children/">Prev 100</a>
             <h3>Page 2 of 2</h3>
           </div>
           <ul>
             <li><a href="/children/{pk}">Test 101</a></li>
           </ul>
        """.format(pk=child.pk))

    def test_template_limit(self):
        for i in range(3, 101):
            child = Child.objects.create(
                name="Test %s" % i,
                parent_id=1,
            )
        html = """
            <p>100 Records</p>
            <div>
              <h3>Page 1 of 10</h3>
              <a href="http://testserver/children/?limit=10&page=2">Next 10</a>
            </div>
            <ul>
        """
        for child in Child.objects.all()[:10]:
            html += """
                <li><a href="/children/{pk}">{label}</a></li>
            """.format(pk=child.pk, label=child.name)
        html += """
           </ul>
        """
        self.check_html("/children/?limit=10", html)

    def test_template_context_processors(self):
        response = self.client.get('/rest_context')
        html = response.content.decode('utf-8')
        token = html.split('value="')[1].split('"')[0]
        self.assertTrue(len(token) >= 32)
        if settings.WITH_NONROOT:
            base_url = '/wqsite'
        else:
            base_url = ''
        self.assertHTMLEqual("""
            <p>{base_url}/rest_context</p>
            <p>rest_context</p>
            <p>{base_url}/</p>
            <p>{base_url}/</p>
            <p>0.0.0</p>
            <p>
              <input name="csrfmiddlewaretoken" type="hidden" value="{csrf}">
            </p>
            <p>rest_context</p>
            <p>Can Edit Items</p>
        """.format(csrf=token, base_url=base_url), html, auto_replace=False)

    def test_template_page_config(self):
        item = Item.objects.get(name="Test 1")
        self.check_html('/items/%s' % item.pk, """
            <h3>Test 1</h3>
            <a href="/itemtypes/1">Test</a>
            <a href="/items/{pk}/edit">Edit</a>
        """.format(pk=item.pk))

    def test_template_edit_fk(self):
        item = Item.objects.get(name="Test 1")
        self.check_html('/items/%s/edit' % item.pk, """
            <form>
              <input name="name" required value="Test 1">
              <select name="type_id" required>
                <option value="">Select one...</option>
                <option value="1" selected>Test</option>
                <option value="2">Inactive</option>
              </select>
              <button>Submit</button>
            </form>
        """)

    def test_template_edit_choice(self):
        self.check_html('/choicemodels/1/edit', """
            <form>
              <input name="name" required value="Test">
              <fieldset>
                <legend>Choice</legend>
                <input type="radio" id="choicemodel-choice-one"
                       name="choice" value="one">
                <label for="choicemodel-choice-one">Choice One</label>
                <input type="radio" id="choicemodel-choice-two"
                       name="choice" value="two" checked>
                <label for="choicemodel-choice-two">Choice Two</label>
                <input type="radio" id="choicemodel-choice-three"
                       name="choice" value="three">
                <label for="choicemodel-choice-three">Choice Three</label>
              </fieldset>
              <button>Submit</button>
            </form>
        """)

    def test_template_new_fk(self):
        self.check_html('/children/new', """
            <form>
              <input name="name" required value="">
              <select name="parent_id" required>
                <option value="">Select one...</option>
                <option value="1">Test</option>
              </select>
              <button>Submit</button>
            </form>
        """)

    def test_template_new_fk_filtered(self):
        self.check_html('/items/new', """
            <form>
              <input name="name" required value="">
              <select name="type_id" required>
                <option value="">Select one...</option>
                <option value="1">Test</option>
              </select>
              <button>Submit</button>
            </form>
        """)

    def test_template_new_fk_defaults(self):
        self.check_html('/items/new?type_id=1', """
            <form>
              <input name="name" required value="">
              <select name="type_id" required>
                <option value="">Select one...</option>
                <option value="1" selected>Test</option>
              </select>
              <button>Submit</button>
            </form>
        """)

    def test_template_new_fk_slug(self):
        self.check_html('/slugrefparents/new?ref_id=test', """
            <form>
              <input name="name" required value="">
              <select name="ref_id" required>
                <option value="">Select one...</option>
                <option value="test" selected>Test</option>
                <option value="other">Other</option>
              </select>
              <button>Submit</button>
            </form>
        """)

    def test_template_new_fk_slug_filtered(self):
        self.check_html('/slugrefchildren/new', """
            <form>
              <input name="name" required value="">
              <select name="parent_id" required>
                <option value="">Select one...</option>
                <option value="1">Test Slug Ref (Test)</option>
              </select>
              <button>Submit</button>
            </form>
        """)

    def test_template_new_choice(self):
        self.check_html('/choicemodels/new', """
            <form>
              <input name="name" required value="">
              <fieldset>
                <legend>Choice</legend>
                <input type="radio" id="choicemodel-choice-one"
                       name="choice" value="one">
                <label for="choicemodel-choice-one">Choice One</label>
                <input type="radio" id="choicemodel-choice-two"
                       name="choice" value="two">
                <label for="choicemodel-choice-two">Choice Two</label>
                <input type="radio" id="choicemodel-choice-three"
                       name="choice" value="three">
                <label for="choicemodel-choice-three">Choice Three</label>
              </fieldset>
              <button>Submit</button>
            </form>
        """)

    def test_template_new_choice_defaults(self):
        self.check_html('/choicemodels/new?choice=three', """
            <form>
              <input name="name" required value="">
              <fieldset>
                <legend>Choice</legend>
                <input type="radio" id="choicemodel-choice-one"
                       name="choice" value="one">
                <label for="choicemodel-choice-one">Choice One</label>
                <input type="radio" id="choicemodel-choice-two"
                       name="choice" value="two">
                <label for="choicemodel-choice-two">Choice Two</label>
                <input type="radio" id="choicemodel-choice-three"
                       name="choice" value="three" checked>
                <label for="choicemodel-choice-three">Choice Three</label>
              </fieldset>
              <button>Submit</button>
            </form>
        """)

    def test_script_tags(self):
        self.check_html('/script_context', """
            <html>
              <head>
                <script async>TEST</script>
                <script async src="chunk1.js"></script>
                <script async src="chunk2.js"></script>
              </head>
              <body>
              </body>
            </html>
        """)



# Django settings
context_processors = (
    'django.template.context_processors.csrf',
    'django.contrib.auth.context_processors.auth',
    'django.contrib.messages.context_processors.messages',
    'wq.db.rest.auth.context_processors.is_authenticated',
    'wq.db.rest.context_processors.version',
    'wq.db.rest.context_processors.router_info',
    'wq.db.rest.context_processors.wq_config',
    'wq.db.rest.context_processors.script_tags',
)

TEMPLATES = [
    {
        'BACKEND': 'django_mustache.Mustache',
        'DIRS': tuple(),
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': context_processors,
        }
    },
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': context_processors,
        }
    }
]


SESSION_COOKIE_HTTPONLY = False

# Django Rest Framework settings
REST_FRAMEWORK = {

    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.TemplateHTMLRenderer',
        'wq.db.rest.renderers.JSONRenderer',
        'wq.db.rest.renderers.GeoJSONRenderer',
    ),

    'DEFAULT_PAGINATION_CLASS': 'wq.db.rest.pagination.Pagination',
    'PAGE_SIZE': 50,

    'DEFAULT_PERMISSION_CLASSES': (
        'wq.db.rest.permissions.ModelPermissions',
    ),

    'DEFAULT_FILTER_BACKENDS': (
        'wq.db.rest.filters.FilterBackend',
    )
}

# wq.db settings
ANONYMOUS_PERMISSIONS = tuple()
SRID = 4326

# Deprecated, do not use
# FIXME: Remove in wq.db 2.0
SOCIAL_AUTH_PIPELINE = None
DEFAULT_AUTH_GROUP = None
DISAMBIGUATE = None



from django.http import Http404
from wq.db.rest.views import ModelViewSet


class IdentifiedModelViewSet(ModelViewSet):
    def get_object(self):
        try:
            obj = super(ModelViewSet, self).get_object()
        except Http404 as notfound:
            # Allow retrieval via non-primary identifiers
            slug = self.kwargs.get(self.lookup_url_kwarg or self.lookup_field)
            try:
                obj = self.model.objects.get_by_identifier(slug)
            except self.model.DoesNotExist:
                raise notfound
            # TODO: automatically redirect to primary identifier?
        return obj



from django.conf import settings
from wq.db.patterns.models import LabelModel


if settings.WITH_GIS:
    from django.contrib.gis.db import models

    class GeometryModel(LabelModel):
        name = models.CharField(max_length=255)
        geometry = models.GeometryField(srid=settings.SRID)

    class PointModel(LabelModel):
        name = models.CharField(max_length=255)
        geometry = models.PointField(srid=settings.SRID)

else:
    GeometryModel = None
    PointModel = None



# Generated by Django 2.1.7 on 2019-03-21 06:12

from django.db import migrations, models
from django.conf import settings


if settings.WITH_GIS:
    import django.contrib.gis.db.models.fields

    class Migration(migrations.Migration):

        initial = True

        dependencies = [
        ]

        operations = [
            migrations.CreateModel(
                name='GeometryModel',
                fields=[
                    ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                    ('name', models.CharField(max_length=255)),
                    ('geometry', django.contrib.gis.db.models.fields.GeometryField(srid=4326)),
                ],
                options={
                    'abstract': False,
                },
            ),
            migrations.CreateModel(
                name='PointModel',
                fields=[
                    ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                    ('name', models.CharField(max_length=255)),
                    ('geometry', django.contrib.gis.db.models.fields.PointField(srid=4326)),
                ],
                options={
                    'abstract': False,
                },
            ),
        ]
else:
    Migration = None



from django.db import models
from natural_keys.models import NaturalKeyModel


class NaturalKeyParent(NaturalKeyModel):
    slug = models.SlugField()

    def __str__(self):
        return self.slug

    class Meta:
        unique_together = [['slug']]


class NaturalKeyChild(NaturalKeyModel):
    date = models.DateField()
    parent = models.ForeignKey(NaturalKeyParent, models.CASCADE)

    def __str__(self):
        return "%s on %s" % (self.parent, self.date)

    class Meta:
        unique_together = [['parent', 'date']]


class ModelWithNaturalKey(models.Model):
    key = models.ForeignKey(NaturalKeyChild, models.CASCADE)
    note = models.TextField()

    def __str__(self):
        return "%s: %s" % (self.key, self.note)
